@@ -65,6 +65,14 @@ int check_ploop_size(unsigned long size);
 #ifdef HAVE_PLOOP
 extern struct ploop_functions ploop;
 
+#define PLOOP_CLEANUP(code)					\
+do {								\
+	struct vzctl_cleanup_handler *__cl_h;			\
+	__cl_h = add_cleanup_handler(cancel_ploop_op, NULL);	\
+	code;							\
+	del_cleanup_handler(__cl_h);				\
+} while(0)
+
 int is_image_mounted(const char *ve_private);
 int vzctl_create_image(const char *ve_private, struct vzctl_create_image_param *param);
 int vzctl_mount_image(const char *ve_private, struct vzctl_mount_param *param);
@@ -75,6 +83,8 @@ int vzctl_get_ploop_dev(const char *mnt, char *out, int len);
 int vzctl_create_snapshot(const char *ve_private, const char *guid);
 int vzctl_merge_snapshot(const char *ve_private, const char *guid);
 int vzctl_delete_snapshot(const char *ve_private, const char *guid);
+int vzctl_mount_snapshot(unsigned envid, const char *ve_private, struct vzctl_mount_param *param);
+int vzctl_umount_snapshot(unsigned envid, const char *ve_private, char *guid);
 const char *generate_snapshot_component_name(unsigned int envid,
 		const char *data, char *buf, int len);
 int vzctl_env_convert_ploop(vps_handler *h, envid_t veid,
