@@ -47,6 +47,8 @@ typedef enum {
 	ACTION_SNAPSHOT_DELETE,
 	ACTION_SNAPSHOT_LIST,
 	ACTION_COMPACT,
+	ACTION_SNAPSHOT_MOUNT,
+	ACTION_SNAPSHOT_UMOUNT,
 #endif
 } act_t;
 
