@@ -56,6 +56,20 @@ static int controller_apply_config(struct cgroup *ct, struct cgroup *parent,
 	} else if (!strcmp("memory", name)) {
 		if ((ret = cgroup_set_value_string(controller, "memory.use_hierarchy", "1")))
 			return ret;
+		/*
+		 * The kernel memory controller cannot flip states from
+		 * unlimited to limited if there are already tasks in it.
+		 * Therefore, we always have to run with *some* value of kmem
+		 * enabled. If we don't do it, we can't start unlimited and
+		 * then use the set command to set any beancounters. We write
+		 * The maximum amount minus two pages, which should effectively
+		 * mean "accounting turned on, but unlimited". This will fail
+		 * if the kmem controller is not present, but that is okay.
+		 */
+		cgroup_set_value_string(controller,
+				"memory.kmem.limit_in_bytes",
+				"9223372036854767712");
+
 	} else if (!strcmp("devices", name)) {
 		if ((ret = cgroup_set_value_string(controller, "devices.deny", "a")))
 			return ret;
@@ -105,7 +119,7 @@ int container_apply_config(envid_t veid, enum conf_files c, void *_val)
 	case SWAP:
 		/* Unlike kmem, this must always be greater than mem */
 		if ((mem = cgroup_add_controller(ct, "memory"))) {
-			unsigned long mval;
+			u_int64_t mval;
 			if (!cgroup_get_value_uint64(mem, MEMLIMIT, &mval))
 				ret = cgroup_set_value_uint64(mem, SWAPLIMIT,
 							      mval + *val);
@@ -317,15 +331,15 @@ int container_add_task(envid_t veid)
 {
 	char cgrp[CT_MAX_STR_SIZE];
 	struct cgroup *ct;
-	int ret = -1;
+	int ret;
 
 	veid_to_name(cgrp, veid);
 	ct = cgroup_new_cgroup(cgrp);
-	if (cgroup_get_cgroup(ct))
+	ret = cgroup_get_cgroup(ct);
+	if (ret)
 		goto out;
 
-	cgroup_attach_task_pid(ct, getpid());
-	ret = 0;
+	ret = cgroup_attach_task_pid(ct, getpid());
 out:
 	cgroup_free(&ct);
 	return ret;
