@@ -40,6 +40,7 @@
 #include "create.h"
 #include "destroy.h"
 #include "image.h"
+#include "cleanup.h"
 
 #define VPS_CREATE	SCRIPTDIR "/vps-create"
 #define VPS_DOWNLOAD	SCRIPTDIR "/vps-download"
@@ -60,7 +61,7 @@ static char *get_ostemplate_name(char *ostmpl)
 		return NULL;
 	}
 	*buf = 0;
-	while((p = fgets(buf, sizeof(buf), fd)) != NULL);
+	while((fgets(buf, sizeof(buf), fd)) != NULL);
 	status = pclose(fd);
 	if (WEXITSTATUS(status) || *buf == 0) {
 		logger(-1, 0, "Unable to get full ostemplate name for %s",
@@ -90,21 +91,59 @@ static int download_template(char *tmpl)
 	return run_script(VPS_DOWNLOAD, arg, env, 0);
 }
 
-static int fs_create(envid_t veid, fs_param *fs, tmpl_param *tmpl,
-	dq_param *dq, int layout, int ploop_mode)
+struct destroy_ve {
+	envid_t veid;
+	char *private;
+};
+
+static void cleanup_destroy_ve(void *data)
+{
+	struct destroy_ve *d = data;
+
+	vps_destroy_dir(d->veid, d->private);
+}
+
+static int fs_create(envid_t veid, vps_handler *h, vps_param *vps_p)
 {
 	char tarball[PATH_LEN];
 	char tmp_dir[PATH_LEN];
 	char buf[PATH_LEN];
 	int ret;
 	char *arg[2];
-	char *env[4];
+	char *env[6];
 	int quota = 0;
 	int i;
 	char *dst;
 	const char *ext[] = { "", ".gz", ".bz2", ".xz", NULL };
 	const char *errmsg_ext = "[.gz|.bz2|.xz]";
+	dq_param *dq = &vps_p->res.dq;
+	int layout = vps_p->opt.layout;
+	fs_param *fs = &vps_p->res.fs;
+	tmpl_param *tmpl = &vps_p->res.tmpl;
+	unsigned long uid_offset = 0;
+	unsigned long gid_offset = 0;
 	int ploop = (layout == VE_LAYOUT_PLOOP);
+	struct destroy_ve ddata;
+	struct vzctl_cleanup_handler *ch;
+
+	/*
+	 * All other users will test directly for h->can_join_userns.  Create
+	 * is special, because we still don't have the container config file
+	 * yet, and the user may be requesting it to be disabled in the command
+	 * line. So that value may be outdated.
+	 *
+	 * By now cmd_p is already merged into vps_p. So what we need to do is
+	 * just to test it again. We will force it to false if it is disabled
+	 * here, or keep the old value otherwise.
+	 */
+	if (!(vps_p->res.misc.local_uid) || (!(*vps_p->res.misc.local_uid)))
+		h->can_join_userns = 0;
+
+	if (h->can_join_userns && vps_p->res.misc.local_uid) {
+		uid_offset = *vps_p->res.misc.local_uid;
+		if (uid_offset && vps_p->res.misc.local_gid)
+			gid_offset = *vps_p->res.misc.local_gid;
+	}
 
 	if (ploop && (!dq->diskspace || dq->diskspace[1] <= 0)) {
 		logger(-1, 0, "Error: diskspace not set (required for ploop)");
@@ -145,6 +184,11 @@ find:
 		goto err;
 	}
 	dst = tmp_dir;
+
+	ddata.veid = veid;
+	ddata.private = dst;
+	ch = add_cleanup_handler(cleanup_destroy_ve, &ddata);
+
 	if (ploop) {
 #ifndef HAVE_PLOOP
 		ret = VZ_PLOOP_UNSUP;
@@ -153,6 +197,7 @@ find:
 		/* Create and mount ploop image */
 		struct vzctl_create_image_param param = {};
 		struct vzctl_mount_param mount_param = {};
+		int ploop_mode = vps_p->opt.mode;
 
 		if (ploop_mode < 0)
 			ploop_mode = PLOOP_EXPANDED_MODE;
@@ -191,11 +236,18 @@ find:
 	arg[0] = VPS_CREATE;
 	arg[1] = NULL;
 	snprintf(buf, sizeof(buf), "PRIVATE_TEMPLATE=%s", tarball);
-	env[0] = strdup(buf);
+	i = 0;
+	env[i++] = strdup(buf);
 	snprintf(buf, sizeof(buf), "VE_PRVT=%s", dst);
-	env[1] = strdup(buf);
-	env[2] = strdup(ENV_PATH);
-	env[3] = NULL;
+	env[i++] = strdup(buf);
+	if (!is_vz_kernel(h) && h->can_join_userns) {
+		snprintf(buf, sizeof(buf), "UID_OFFSET=%lu", uid_offset);
+		env[i++] = strdup(buf);
+		snprintf(buf, sizeof(buf), "GID_OFFSET=%lu", gid_offset);
+		env[i++] = strdup(buf);
+	}
+	env[i++] = strdup(ENV_PATH);
+	env[i] = NULL;
 	logger(0, 0, "Creating container private area (%s)", tmpl->ostmpl);
 	ret = run_script(VPS_CREATE, arg, env, 0);
 	free_arg(env);
@@ -214,6 +266,7 @@ find:
 	}
 	/* Unlock CT area */
 	rmdir(fs->private);
+	del_cleanup_handler(ch);
 	if (rename(tmp_dir, fs->private)) {
 		logger(-1, errno, "Can't rename %s to %s", tmp_dir, fs->private);
 		ret = VZ_FS_NEW_VE_PRVT;
@@ -230,6 +283,11 @@ err:
 	return ret;
 }
 
+static void cleanup_destroy_file(void *data)
+{
+	unlink(data);
+}
+
 int vps_create(vps_handler *h, envid_t veid, vps_param *vps_p, vps_param *cmd_p,
 	struct mod_action *action)
 {
@@ -242,6 +300,7 @@ int vps_create(vps_handler *h, envid_t veid, vps_param *vps_p, vps_param *cmd_p,
 	vps_param *conf_p;
 	int cfg_exists;
 	char *full_ostmpl;
+	struct vzctl_cleanup_handler *ch = NULL;
 
 	get_vps_conf_path(veid, dst, sizeof(dst));
 	sample_config = NULL;
@@ -288,6 +347,7 @@ int vps_create(vps_handler *h, envid_t veid, vps_param *vps_p, vps_param *cmd_p,
 		}
 	}
 	if (sample_config != NULL) {
+		ch = add_cleanup_handler(cleanup_destroy_file, &dst);
 		if (cp_file(dst, src))
 		{
 			ret = VZ_CP_CONFIG;
@@ -360,9 +420,8 @@ int vps_create(vps_handler *h, envid_t veid, vps_param *vps_p, vps_param *cmd_p,
 				tmpl->ostmpl = full_ostmpl;
 			}
 		}
-		if ((ret = fs_create(veid, fs, tmpl, &vps_p->res.dq,
-						vps_p->opt.layout,
-						vps_p->opt.mode)))
+		ret = fs_create(veid, h, vps_p);
+		if (ret)
 			goto err_root;
 	}
 
@@ -396,6 +455,7 @@ int vps_create(vps_handler *h, envid_t veid, vps_param *vps_p, vps_param *cmd_p,
 		goto err_names;
 	}
 
+	del_cleanup_handler(ch);
 	logger(0, 0, "Container private area was created");
 	return 0;
 
