@@ -153,12 +153,7 @@ int vps_mount(vps_handler *h, envid_t veid, fs_param *fs, dq_param *dq,
 		return VZ_VE_ROOT_NOTSET;
 	if (check_var(fs->private, "VE_PRIVATE is not set"))
 		return VZ_VE_PRIVATE_NOTSET;
-	if (!stat_file(fs->private)) {
-		logger(-1, 0, "Container private area %s does not exist",
-				fs->private);
-		return VZ_FS_NOPRVT;
-	}
-	if (vps_is_mounted(fs->root, fs->private)) {
+	if (vps_is_mounted(fs->root, fs->private) == 1) {
 		logger(-1, 0, "Container is already mounted");
 		return 0;
 	}
@@ -168,14 +163,24 @@ int vps_mount(vps_handler *h, envid_t veid, fs_param *fs, dq_param *dq,
 			PRE_MOUNT_PREFIX);
 		for (i = 0; i < 2; i++) {
 			if (run_pre_script(veid, buf)) {
-				logger(-1, 0, "Error executing mount script %s",
-					buf);
+				logger(-1, 0, "Error executing "
+						"premount script %s", buf);
 				return VZ_ACTIONSCRIPT_ERROR;
 			}
 			snprintf(buf, sizeof(buf), "%s%d.%s", VPS_CONF_DIR,
 				veid, PRE_MOUNT_PREFIX);
 		}
 	}
+	if (!stat_file(fs->private)) {
+		logger(-1, 0, "Container private area %s does not exist",
+				fs->private);
+		return VZ_FS_NOPRVT;
+	}
+	if (!stat_file(fs->private)) {
+		logger(-1, 0, "Container private area %s does not exist",
+				fs->private);
+		return VZ_FS_NOPRVT;
+	}
 	if ((ret = fsmount(veid, fs, dq)))
 		return ret;
 	/* Execute per-CT & global mount scripts */
@@ -204,7 +209,7 @@ int vps_umount(vps_handler *h, envid_t veid, const fs_param *fs,
 	char buf[PATH_LEN];
 	int ret, i;
 
-	if (!vps_is_mounted(fs->root, fs->private)) {
+	if (vps_is_mounted(fs->root, fs->private) == 0) {
 		logger(-1, 0, "CT is not mounted");
 		return VZ_FS_NOT_MOUNTED;
 	}
@@ -232,8 +237,8 @@ int vps_umount(vps_handler *h, envid_t veid, const fs_param *fs,
 			veid, POST_UMOUNT_PREFIX);
 		for (i = 0; i < 2; i++) {
 			if (run_pre_script(veid, buf)) {
-				logger(-1, 0, "Error executing umount script %s",
-					buf);
+				logger(-1, 0, "Error executing "
+						"postumount script %s",	buf);
 				return VZ_ACTIONSCRIPT_ERROR;
 			}
 			snprintf(buf, sizeof(buf), "%svps.%s", VPS_CONF_DIR,
@@ -253,7 +258,7 @@ int vps_set_fs(fs_param *g_fs, fs_param *fs)
 		return VZ_VE_ROOT_NOTSET;
 	if (check_var(g_fs->private, "VE_PRIVATE is not set"))
 		return VZ_VE_PRIVATE_NOTSET;
-	if (!vps_is_mounted(g_fs->root, g_fs->private)) {
+	if (vps_is_mounted(g_fs->root, g_fs->private) == 0) {
 		logger(-1, 0, "Container is not mounted");
 		return VZ_FS_NOT_MOUNTED;
 	}
