@@ -36,6 +36,7 @@
 #include "vzctl.h"
 #include "env.h"
 #include "destroy.h"
+#include "cleanup.h"
 
 #define DEFAULT_FSTYPE		"ext4"
 #define SNAPSHOT_MOUNT_ID	"snap"
@@ -105,7 +106,7 @@ static int load_ploop_lib(void)
 		 * to prevent a buffer overflow.
 		 */
 		void *padding[32];
-	} src;
+	} src = { {0}, {0} };
 
 	h = dlopen("libploop.so", RTLD_LAZY);
 	if (!h) {
@@ -119,6 +120,7 @@ static int load_ploop_lib(void)
 	if ((err = dlerror()) != NULL) {
 		logger(-1, 0, "Can't init ploop library: %s", err);
 		logger(-1, 0, "Please upgrade your ploop packages!");
+		dlclose(h);
 		return -1;
 	}
 
@@ -129,6 +131,7 @@ static int load_ploop_lib(void)
 
 	vzctl_init_ploop_log();
 	logger(1, 0, "The ploop library has been loaded successfully");
+	/* Don't dlclose(), we need lib for the rest of runtime */
 
 	return 0;
 }
@@ -166,6 +169,11 @@ int is_ploop_supported()
 
 #ifdef HAVE_PLOOP
 
+static void cancel_ploop_op(void *data)
+{
+	ploop.cancel_operation();
+}
+
 /* Note: caller should call is_ploop_supported() before.
  * Currently the only caller is vps_start_custom() which does.
  */
@@ -210,7 +218,7 @@ int vzctl_mount_image(const char *ve_private, struct vzctl_mount_param *param)
 	mount_param.quota = param->quota;
 	mount_param.mount_data = param->mount_data;
 
-	ret = ploop.mount_image(di, &mount_param);
+	PLOOP_CLEANUP(ret = ploop.mount_image(di, &mount_param));
 	if (ret) {
 		logger(-1, 0, "Failed to mount image: %s [%d]",
 				ploop.get_last_error(), ret);
@@ -236,7 +244,7 @@ int vzctl_umount_image(const char *ve_private)
 		return VZCTL_E_UMOUNT_IMAGE;
 	}
 
-	ret = ploop.umount_image(di);
+	PLOOP_CLEANUP(ret = ploop.umount_image(di));
 	if (ret) {
 		logger(-1, 0, "Failed to umount image: %s [%d]",
 				ploop.get_last_error(), ret);
@@ -269,7 +277,7 @@ int vzctl_create_image(const char *ve_private,
 	create_param.fstype = DEFAULT_FSTYPE;
 	create_param.size = param->size * 2; /* Kb to 512b sectors */
 	create_param.image = image;
-	ret = ploop.create_image(&create_param);
+	PLOOP_CLEANUP(ret = ploop.create_image(&create_param));
 	if (ret) {
 		rmdir(dir);
 		logger(-1, 0, "Failed to create image: %s [%d]",
@@ -294,7 +302,7 @@ int vzctl_convert_image(const char *ve_private, int mode)
 		logger(-1, 0, "Failed to read %s", fname);
 		return VZCTL_E_CONVERT_IMAGE;
 	}
-	ret = ploop.convert_image(di, mode, 0);
+	PLOOP_CLEANUP(ret = ploop.convert_image(di, mode, 0));
 	if (ret) {
 		logger(-1, 0, "Failed to convert image: %s [%d]",
 				ploop.get_last_error(), ret);
@@ -330,7 +338,7 @@ int vzctl_resize_image(const char *ve_private, unsigned long long newsize)
 		return VZCTL_E_RESIZE_IMAGE;
 	}
 	param.size = newsize * 2; /* Kb to 512b sectors */
-	ret = ploop.resize_image(di, &param);
+	PLOOP_CLEANUP(ret = ploop.resize_image(di, &param));
 	if (ret) {
 		logger(-1, 0, "Failed to resize image: %s [%d]",
 				ploop.get_last_error(), ret);
@@ -369,7 +377,7 @@ int vzctl_create_snapshot(const char *ve_private, const char *guid)
 		return VZCTL_E_CREATE_SNAPSHOT;
 	}
 	param.guid = strdup(guid);
-	ret = ploop.create_snapshot(di, &param);
+	PLOOP_CLEANUP(ret = ploop.create_snapshot(di, &param));
 	free(param.guid);
 	if (ret) {
 		logger(-1, 0, "Failed to create snapshot: %s [%d]",
@@ -400,7 +408,7 @@ int vzctl_delete_snapshot(const char *ve_private, const char *guid)
 		logger(-1, 0, "Failed to read %s", fname);
 		return VZCTL_E_DELETE_SNAPSHOT;
 	}
-	ret = ploop.delete_snapshot(di, guid);
+	PLOOP_CLEANUP(ret = ploop.delete_snapshot(di, guid));
 	if (ret) {
 		logger(-1, 0, "Failed to delete snapshot: %s [%d]",
 				ploop.get_last_error(), ret);
@@ -434,7 +442,7 @@ int vzctl_merge_snapshot(const char *ve_private, const char *guid)
 		return VZCTL_E_MERGE_SNAPSHOT;
 	}
 	param.guid = guid;
-	ret = ploop.merge_snapshot(di, &param);
+	PLOOP_CLEANUP(ret = ploop.merge_snapshot(di, &param));
 	if (ret) {
 		logger(-1, 0, "Failed to merge snapshot %s: %s [%d]",
 				guid, ploop.get_last_error(), ret);
@@ -454,6 +462,85 @@ const char *generate_snapshot_component_name(unsigned int envid,
 	return buf;
 }
 
+int vzctl_mount_snapshot(unsigned envid, const char *ve_private,
+		struct vzctl_mount_param *param)
+{
+	int ret = VZCTL_E_MOUNT_SNAPSHOT;
+	char fname[PATH_MAX];
+	struct ploop_disk_images_data *di;
+	struct ploop_mount_param mount_param = {};
+
+	if (!is_ploop_supported())
+		return VZ_PLOOP_UNSUP;
+
+	GET_DISK_DESCRIPTOR(fname, ve_private);
+	if (ploop.read_disk_descr(&di, fname)) {
+		logger(-1, 0, "Failed to read %s", fname);
+		return ret;
+	}
+
+	mount_param.ro = param->ro;
+	if (param->mount_by_parent_guid) {
+		mount_param.guid = ploop.find_parent_by_guid(di, param->guid);
+		if (mount_param.guid == NULL) {
+			logger(-1, 0, "Unable to find parent guid by %s",
+					param->guid);
+			goto err;
+		}
+	} else {
+		mount_param.guid = (char *)param->guid;
+	}
+	mount_param.target = param->target;
+	ploop.set_component_name(di,
+			generate_snapshot_component_name(envid, param->guid,
+				fname, sizeof(fname)));
+
+	PLOOP_CLEANUP(ret = ploop.mount_image(di, &mount_param));
+	if (ret) {
+		logger(-1, 0, "Failed to mount snapshot %s: %s [%d]",
+				param->guid, ploop.get_last_error(), ret);
+		ret = VZCTL_E_MOUNT_SNAPSHOT;
+		goto err;
+	}
+
+	/* provide device to caller */
+	strncpy(param->device, mount_param.device, sizeof(param->device)-1);
+
+err:
+	ploop.free_diskdescriptor(di);
+
+	return ret;
+}
+
+int vzctl_umount_snapshot(unsigned envid, const char *ve_private, char *guid)
+{
+	int ret;
+	char fname[PATH_MAX];
+	struct ploop_disk_images_data *di;
+
+	if (!is_ploop_supported())
+		return VZ_PLOOP_UNSUP;
+
+	GET_DISK_DESCRIPTOR(fname, ve_private);
+	if (ploop.read_disk_descr(&di, fname)) {
+		logger(-1, 0, "Failed to read %s", fname);
+		return VZCTL_E_UMOUNT_SNAPSHOT;
+	}
+
+	ploop.set_component_name(di,
+			generate_snapshot_component_name(envid, guid,
+				fname, sizeof(fname)));
+
+	PLOOP_CLEANUP(ret = ploop.umount_image(di));
+	ploop.free_diskdescriptor(di);
+	if (ret)
+		return vzctl_err(VZCTL_E_UMOUNT_SNAPSHOT, 0,
+				"Failed to umount snapshot %s: %s [%d]",
+				guid, ploop.get_last_error(), ret);
+
+	return 0;
+}
+
 /* Convert a CT to ploop layout
  * 1) mount CT
  * 2) create & mount image
@@ -480,7 +567,7 @@ int vzctl_env_convert_ploop(vps_handler *h, envid_t veid,
 		logger(-1, 0, "CT is running (stop it first)");
 		return VZ_VE_RUNNING;
 	}
-	if (vps_is_mounted(fs->root, fs->private)) {
+	if (vps_is_mounted(fs->root, fs->private) == 1) {
 		logger(-1, 0, "CT is mounted (umount it first)");
 		return VZ_FS_MOUNTED;
 	}
