@@ -58,10 +58,14 @@ static int fill_2quota_param(struct setup_env_quota_param *p,
 	}
 
 	/* ploop case */
+	if (!is_ploop_supported())
+		return VZ_PLOOP_UNSUP;
+#ifdef HAVE_PLOOP
 	if (vzctl_get_ploop_dev(ve_root, p->dev_name, sizeof(p->dev_name))) {
 		logger(-1, 0, "Unable to find ploop device for %s", ve_root);
 		return VZ_ERROR_SET_USER_QUOTA;
 	}
+#endif
 	if (stat(p->dev_name, &st)) {
 		logger(-1, errno, "%s: Can't stat %s", __func__, p->dev_name);
 		return VZ_ERROR_SET_USER_QUOTA;
@@ -81,7 +85,7 @@ static int vps_2quota_perm(vps_handler *h, int veid, dev_t device)
 	dev.dev = device;
 	dev.type = S_IFBLK | VE_USE_MINOR;
 	dev.mask = S_IXGRP;
-	return set_devperm(h, veid, &dev);
+	return h->setdevperm(h, veid, &dev);
 }
 
 int vps_setup_res(vps_handler *h, envid_t veid, dist_actions *actions,
