@@ -37,6 +37,8 @@
 #include "script.h"
 #include "fs.h"
 #include "dev.h"
+#include "exec.h"
+#include "cleanup.h"
 
 volatile sig_atomic_t alarm_flag;
 static char *envp_bash[] = {"HOME=/", "TERM=linux", ENV_PATH, NULL};
@@ -61,7 +63,7 @@ int read_script(const char *fname, char *include, char **buf)
 			snprintf(inc, p - fname + 2, "%s", fname);
 			strcat(inc, include);
 		} else {
-			snprintf(inc, sizeof(inc), "%s", include);
+			sprintf(inc, "%s", include);
 		}
 		if (stat_file(inc))
 			len = read_script(inc, NULL, buf);
@@ -100,7 +102,7 @@ int read_script(const char *fname, char *include, char **buf)
 
 	return len;
 err:
-	if (fd > 0)
+	if (fd >= 0)
 		close(fd);
 	free(*buf);
 	return -1;
@@ -109,13 +111,12 @@ err:
 #define ENV_SIZE	256
 int run_script(const char *f, char *argv[], char *env[], int quiet)
 {
-	int child, pid, fd;
-	int status;
+	int child, fd;
 	int ret, i, j;
 	char *cmd;
 	struct sigaction act, actold;
-	int out[2];
 	char *envp[ENV_SIZE];
+	struct vzctl_cleanup_handler *ch;
 
 	if (!stat_file(f)) {
 		logger(-1, 0, "File %s not found", f);
@@ -132,11 +133,6 @@ int run_script(const char *f, char *argv[], char *env[], int quiet)
 		logger(2, 0, "Running: %s", cmd);
 		free(cmd);
 	}
-	if (quiet && pipe(out) < 0) {
-		logger(-1, errno, "run_script: unable to create pipe");
-		ret = VZ_RESOURCE_ERROR;
-		goto err;
-	}
 	i = 0;
 	for (i = 0; i < ENV_SIZE - 1 && envp_bash[i] != NULL; i++)
 			envp[i] = envp_bash[i];
@@ -151,16 +147,9 @@ int run_script(const char *f, char *argv[], char *env[], int quiet)
 		else
 			dup2(fd, 0);
 
-		if (quiet) {
+		if (quiet && fd >= 0) {
 			dup2(fd, 1);
 			dup2(fd, 2);
-		} else {
-/*
-			dup2(out[1], STDOUT_FILENO);
-			dup2(out[1], STDERR_FILENO);
-			close(out[0]);
-			close(out[1]);
-*/
 		}
 		execve(f, argv, envp);
 		logger(-1, errno, "Error exec %s", f);
@@ -170,19 +159,10 @@ int run_script(const char *f, char *argv[], char *env[], int quiet)
 		ret = VZ_RESOURCE_ERROR;
 		goto err;
 	}
-	while ((pid = waitpid(child, &status, 0)) == -1)
-		if (errno != EINTR)
-			break;
-	ret = VZ_SYSTEM_ERROR;
-	if (pid == child) {
-		if (WIFEXITED(status))
-			ret = WEXITSTATUS(status);
-		else if (WIFSIGNALED(status))
-			logger(-1, 0, "Received signal:"
-					"  %d in %s", WTERMSIG(status), f);
-	} else {
-		logger(-1, errno, "Error in waitpid");
-	}
+	ch = add_cleanup_handler(cleanup_kill_process, &child);
+	ret = env_wait(child);
+	del_cleanup_handler(ch);
+
 err:
 	sigaction(SIGCHLD, &actold, NULL);
 
@@ -295,7 +275,7 @@ int add_reach_runlevel_mark()
 {
 	int fd, found, ret;
 	ssize_t len, w;
-	char buf[4096];
+	char buf[4096 + 1];
 	struct stat st;
 
 	unlink(VZFIFO_FILE);
@@ -349,7 +329,7 @@ int add_reach_runlevel_mark()
 	ret = 0;
 	found = 0;
 	while (1) {
-		len = read(fd, buf, sizeof(buf));
+		len = read(fd, buf, sizeof(buf) - 1);
 		if (len == 0)
 			break;
 		if (len < 0) {
@@ -411,5 +391,7 @@ err:
 	alarm(0);
 	sigaction(SIGALRM, &actold, NULL);
 	unlink(VZFIFO_FILE);
+	if (fd >= 0)
+		close(fd);
 	return ret;
 }
