@@ -45,6 +45,7 @@
 #include "image.h"
 #include "cpt.h"
 #include "snapshot.h"
+#include "cleanup.h"
 
 extern struct mod_action g_action;
 extern int do_enter(vps_handler *h, envid_t veid, const char *root,
@@ -104,7 +105,6 @@ static struct option set_opt[] = {
 	/*	fs param */
 	{"root",	required_argument, NULL, PARAM_ROOT},
 	{"private",	required_argument, NULL, PARAM_PRIVATE},
-	{"noatime",	required_argument, NULL, PARAM_IGNORED},
 	{"mount_opts",	required_argument, NULL, PARAM_MOUNT_OPTS},
 	/*	template	*/
 	{"ostemplate",	required_argument, NULL, PARAM_OSTEMPLATE},
@@ -391,6 +391,8 @@ static int parse_create_opt(envid_t veid, int argc, char **argv,
 	{"ve_layout",	required_argument, NULL, PARAM_VE_LAYOUT},
 	{"velayout",	required_argument, NULL, PARAM_VE_LAYOUT},
 	{"diskspace",	required_argument, NULL, PARAM_DISKSPACE},
+	{"local_uid",	required_argument, NULL, PARAM_LOCAL_UID},
+	{"local_gid",	required_argument, NULL, PARAM_LOCAL_GID},
 	{ NULL, 0, NULL, 0 }
 };
 
@@ -496,7 +498,7 @@ static int compact(vps_handler *h, envid_t veid, vps_param *g_p,
 	}
 
 	mounted = vps_is_mounted(root, private);
-	if (!mounted)
+	if (mounted == 0)
 	{
 		ret = vps_mount(h, veid, &g_p->res.fs, &g_p->res.dq,
 				SKIP_ACTION_SCRIPT);
@@ -513,7 +515,7 @@ static int compact(vps_handler *h, envid_t veid, vps_param *g_p,
 	ret = system(cmd);
 	free(cmd);
 
-	if (!mounted)
+	if (mounted == 0)
 		vps_umount(h, veid, &g_p->res.fs, SKIP_ACTION_SCRIPT);
 
 	return (ret == 0) ? 0 : VZCTL_E_COMPACT_IMAGE;
@@ -541,7 +543,7 @@ static int parse_chkpnt_opt(int argc, char **argv, vps_param *vps_p)
 
 	while (1) {
 		option_index = -1;
-		c = getopt_long (argc, argv, "", chkpnt_options, NULL);
+		c = getopt_long (argc, argv, "", chkpnt_options, &option_index);
 		if (c == -1)
 			break;
 		switch (c) {
@@ -616,7 +618,7 @@ static int parse_restore_opt(int argc, char **argv, vps_param *vps_p)
 
 	while (1) {
 		option_index = -1;
-		c = getopt_long (argc, argv, "", restore_options, NULL);
+		c = getopt_long (argc, argv, "", restore_options, &option_index);
 		if (c == -1)
 			break;
 		switch (c) {
@@ -715,6 +717,36 @@ static int parse_snapshot_delete_opt(envid_t veid, int argc, char **argv,
 
 	return ret;
 }
+
+static int parse_snapshot_mount_opt(envid_t veid, int argc, char **argv,
+		vps_param *param)
+{
+	int ret;
+	struct option *opt;
+	static struct option snapshot_create_options[] =
+	{
+	{"id",  required_argument, NULL, PARAM_SNAPSHOT_GUID},
+	{"uuid", required_argument, NULL, PARAM_SNAPSHOT_GUID},
+	{"target", required_argument, NULL, PARAM_SNAPSHOT_MOUNT_TARGET},
+	{ NULL, 0, NULL, 0 }
+	};
+
+	opt = mod_make_opt(snapshot_create_options, &g_action, NULL);
+	if (opt == NULL)
+		return VZ_RESOURCE_ERROR;
+	ret = parse_opt(veid, argc, argv, opt, snapshot_create_options, param);
+	free(opt);
+
+	if (ret == 0 && param->snap.guid == NULL)
+		return vzctl_err(VZ_INVALID_PARAMETER_SYNTAX, 0,
+			"Invalid syntax: snapshot id is missing");
+	if (ret == 0 && param->snap.target == NULL)
+		return vzctl_err(VZ_INVALID_PARAMETER_SYNTAX, 0,
+			"Invalid syntax: mount target is missing");
+
+	return ret;
+}
+
 #endif /* HAVE_PLOOP */
 
 static int check_set_ugidlimit(unsigned long *cur, unsigned long *old,
@@ -858,6 +890,42 @@ static int apply_param_from_cfg(int veid, vps_param *param, char *cfg)
 	return 0;
 }
 
+static int check_set_opt(int argc, char *argv[], vps_param *param)
+{
+	if ((param->opt.reset_ub == YES) && (argc > 1)) {
+		logger(-1, 0, "Error: option --reset_ub is exclusive");
+		return VZ_INVALID_PARAMETER_SYNTAX;
+	}
+
+	/* A few options require --save flag */
+	if (!param->opt.save) {
+		if (param->res.name.name != NULL) {
+			logger(-1, 0, "Error: unable to use"
+				 " --name option without --save");
+			return VZ_SET_NAME_ERROR;
+		}
+		if (param->opt.save_force) {
+			logger(-1, 0, "Error: --force is useless "
+					"without --save");
+			return VZ_INVALID_PARAMETER_SYNTAX;
+		}
+		if (param->res.dq.diskspace &&
+				ve_private_is_ploop(param->g_param->res.fs.private))
+		{
+			logger(-1, 0, "Error: can't use --diskspace without --save "
+					"for a ploop-based container");
+			return VZ_INVALID_PARAMETER_SYNTAX;
+		}
+		if (param->opt.setmode == SET_RESTART) {
+			logger(-1, 0, "Error: --setmode restart doesn't make sense"
+					" without --save flag");
+			return VZ_INVALID_PARAMETER_SYNTAX;
+		}
+	}
+
+	return 0;
+}
+
 static int parse_set_opt(envid_t veid, int argc, char *argv[],
 	vps_param *param)
 {
@@ -874,7 +942,7 @@ static int parse_set_opt(envid_t veid, int argc, char *argv[],
 }
 
 static int set_ve0(vps_handler *h, vps_param *g_p,
-	vps_param *vps_p, vps_param *cmd_p)
+	vps_param *vps_p, vps_param *cmd_p, int *warn_save)
 {
 	int ret;
 	ub_param *ub;
@@ -883,12 +951,16 @@ static int set_ve0(vps_handler *h, vps_param *g_p,
 	if (cmd_p->opt.reset_ub == YES) {
 		/* Apply parameters from config */
 		ub = &vps_p->res.ub;
-		cmd_p->opt.save = NO; // suppress savewarning
+		*warn_save = 0;
 	} else {
 		/* Apply parameters from command line */
 		ub = &cmd_p->res.ub;
 		cpu = &cmd_p->res.cpu;
 	}
+
+	if (!h)
+		return 0;
+
 	ret = vps_set_ublimit(h, 0, ub);
 	if (ret)
 		return ret;
@@ -927,7 +999,7 @@ static int fix_ip_param(vps_param *conf, vps_param *cmd)
 }
 
 static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
-	vps_param *cmd_p)
+	vps_param *cmd_p, int argc, char **argv, int *warn_save)
 {
 	int ret = 0, is_run;
 	dist_actions *actions = NULL;
@@ -940,12 +1012,16 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 		ret = check_veth_param(veid, &vps_p->res.veth, &cmd_p->res.veth,
 			&cmd_p->del_res.veth);
 		if (ret) {
-			cmd_p->opt.save = NO;
+			*warn_save = 0;
 			return ret;
 		}
 	}
 
 	cmd_p->g_param = g_p;
+	ret = check_set_opt(argc, argv, cmd_p);
+	if (ret)
+		return ret;
+
 	if (cmd_p->opt.apply_cfg_map == APPCONF_MAP_NAME) {
 		ret = set_name(veid, g_p->res.name.name, g_p->res.name.name);
 		if (ret != 0)
@@ -954,25 +1030,14 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 	/* Reset UB parameters from config  */
 	if (cmd_p->opt.reset_ub == YES && h != NULL) {
 		ret = vps_set_ublimit(h, veid, &vps_p->res.ub);
-		cmd_p->opt.save = NO; // suppress savewarning
+		*warn_save = 0;
 		return ret;
 	}
-	if (cmd_p->opt.setmode == SET_RESTART && !cmd_p->opt.save) {
-		logger(-1, 0, "Error: --setmode restart doesn't make sense"
-			" without --save flag");
-		return VZ_INVALID_PARAMETER_SYNTAX;
-	}
 	if (cmd_p->res.name.name != NULL) {
-		if (!cmd_p->opt.save) {
-			logger(-1, 0, "Error: unable to use"
-				 " --name option without --save");
-			ret =  VZ_SET_NAME_ERROR;
-		} else {
-			ret = set_name(veid, cmd_p->res.name.name,
+		ret = set_name(veid, cmd_p->res.name.name,
 				vps_p->res.name.name);
-		}
 		if (ret) {
-			cmd_p->opt.save = NO;
+			*warn_save = 0;
 			return ret;
 		}
 	}
@@ -1001,18 +1066,24 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 		if (!actions)
 			return VZ_RESOURCE_ERROR;
 		dist_name = get_dist_name(&g_p->res.tmpl);
-		if ((ret = read_dist_actions(dist_name, DIST_DIR, actions)))
+		if ((ret = read_dist_actions(dist_name, DIST_DIR, actions))) {
+			free(actions);
 			return ret;
+		}
 		free(dist_name);
 	}
 	/* Setup password */
 	if (h != NULL && !list_empty(&cmd_p->res.misc.userpw)) {
-		if (!is_run)
-			if ((ret = vps_start(h, veid, g_p,
-				SKIP_SETUP|SKIP_ACTION_SCRIPT, NULL)))
-			{
+		if (!is_run) {
+			ret = vps_start(h, veid, g_p,
+					SKIP_SETUP|SKIP_ACTION_SCRIPT, NULL);
+			if (ret)
 				goto err;
-			}
+		}
+
+		if (argc == 2) /* only --userpasswd and its value */
+			*warn_save = 0;
+
 		ret = vps_pw_configure(h, veid, actions, g_p->res.fs.root,
 			&cmd_p->res.misc.userpw);
 		if (!is_run)
@@ -1032,12 +1103,14 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 	if (cmd_p->res.dq.diskspace &&
 			ve_private_is_ploop(g_p->res.fs.private))
 	{
-		if (! is_ploop_supported())
-			return VZ_PLOOP_UNSUP;
+		if (! is_ploop_supported()) {
+			ret=VZ_PLOOP_UNSUP;
+			goto err;
+		}
 #ifdef HAVE_PLOOP
 		if ((ret = vzctl_resize_image(g_p->res.fs.private,
 						cmd_p->res.dq.diskspace[1])))
-			return ret;
+			goto err;
 #endif
 	}
 	/* Warn that --diskinodes is ignored for ploop CT */
@@ -1051,24 +1124,33 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 	if (cmd_p->opt.save && !is_run) {
 		ret = mod_setup(h, veid, STATE_STOPPED, SKIP_NONE, &g_action,
 			cmd_p);
-		return ret;
+		goto err;
 	}
 	if (is_run) {
+		/* Check if restart is required */
 		ret = check_set_mode(h, veid, cmd_p->opt.setmode,
 			cmd_p->opt.save, &cmd_p->res, &g_p->res);
 		if (ret) {
 			if (ret < 0) {
+				/* ignore but return error */
 				ret = VZ_VE_RUNNING;
-				goto err;
 			} else {
+				/* do restart */
 				merge_vps_param(g_p, cmd_p);
 				ret = restart(h, veid, g_p, cmd_p);
-				goto err;
 			}
+			goto err;
 		}
 	}
-	ret = vps_setup_res(h, veid, actions, &g_p->res.fs, cmd_p,
-		STATE_RUNNING, SKIP_NONE, &g_action);
+
+	/* Only try to apply parameters on a supported kernel */
+	if (h) {
+		/* If CT is not running, call this anyway to get relevant
+		 * errors messages like "Can't set CPU: CT is not running"
+		 */
+		ret = vps_setup_res(h, veid, actions, &g_p->res.fs, cmd_p,
+				STATE_RUNNING, SKIP_NONE, &g_action);
+	}
 err:
 	free_dist_actions(actions);
 	free(actions);
@@ -1076,6 +1158,45 @@ err:
 	return ret;
 }
 
+static int vps_set(vps_handler *h, envid_t veid,
+		vps_param *g_p, vps_param *vps_p, vps_param *cmd_p,
+		int argc, char **argv)
+{
+	int ret;
+	int warn_save = 1;
+	char fname[STR_SIZE];
+
+	if (veid == 0)
+		ret = set_ve0(h, g_p, vps_p, cmd_p, &warn_save);
+	else
+		ret = set(h, veid, g_p, vps_p, cmd_p, argc, argv, &warn_save);
+
+	if (cmd_p->opt.save == YES) {
+		if (ret) {
+			logger(-1, 0, "Error: failed to apply "
+					"some parameters, not saving "
+					"configuration file!");
+			return ret;
+		}
+
+		get_vps_conf_path(veid, fname, sizeof(fname));
+		/* Warn if config does not exist */
+		if (!stat_file(fname))
+			logger(-1, errno, "WARNING: %s not found", fname);
+
+		ret = vps_save_config(veid, fname, cmd_p, vps_p, &g_action);
+	}
+	else if (warn_save && !ret) {
+		int is_run = h != NULL && vps_is_run(h, veid);
+		logger(0, 0, "WARNING: Settings were not saved"
+				" to config %s(use --save flag)",
+				is_run ? "and will be lost after CT restart "
+				: "");
+	}
+
+	return ret;
+}
+
 static int enter(vps_handler *h, envid_t veid, const char *root,
 		int argc, char **argv)
 {
@@ -1151,6 +1272,7 @@ static int exec(vps_handler *h, act_t action, envid_t veid, const char *root,
 	}
 	logger(1, 0, "Executing command: %s", buf);
 	ret = vps_exec(h, veid, root, mode, arg, NULL, NULL, 0);
+	free(buf);
 
 	return ret;
 }
@@ -1315,10 +1437,14 @@ int parse_action_opt(envid_t veid, act_t action, int argc, char *argv[],
 		break;
 	case ACTION_SNAPSHOT_DELETE:
 	case ACTION_SNAPSHOT_SWITCH:
+	case ACTION_SNAPSHOT_UMOUNT:
 		ret = parse_snapshot_delete_opt(veid, argc, argv, param);
 		break;
 	case ACTION_SNAPSHOT_LIST:
 		break;
+	case ACTION_SNAPSHOT_MOUNT:
+		ret = parse_snapshot_mount_opt(veid, argc, argv, param);
+		break;
 #endif
 	default :
 		if ((argc - 1) > 0) {
@@ -1332,18 +1458,24 @@ int parse_action_opt(envid_t veid, act_t action, int argc, char *argv[],
 	return ret;
 }
 
+
+static void cleanup_callback(int sig)
+{
+	fprintf(stderr, "\nCancelling...\n");
+	run_cleanup();
+}
+
 int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 	vps_param *cmd_p, int argc, char **argv, int skiplock)
 {
 	vps_handler *h = NULL;
-	int ret, lock_id = -1;
+	int ret = 0, lock_id = -1;
 	struct sigaction act;
-	char fname[STR_SIZE];
 
-	ret = 0;
-	if ((h = vz_open(veid)) == NULL) {
-		/* Accept to run "set --save --force" on non-openvz
-		 * kernel */
+	if ((h = vz_open(veid, g_p)) == NULL) {
+		/* Accept to run "set --save --force" on any kernel,
+		 * otherwise error out if initialization failed
+		 */
 		if (action != ACTION_SET ||
 		    cmd_p->opt.save_force != YES ||
 		    cmd_p->opt.save != YES)
@@ -1378,9 +1510,11 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 				goto err;
 			}
 		}
+
 		sigemptyset(&act.sa_mask);
-		act.sa_handler = SIG_IGN;
+		act.sa_handler = cleanup_callback;
 		act.sa_flags = 0;
+		sigaction(SIGTERM, &act, NULL);
 		sigaction(SIGINT, &act, NULL);
 	}
 	switch (action) {
@@ -1414,33 +1548,7 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 		break;
 #endif
 	case ACTION_SET:
-		if (veid == 0)
-			ret = set_ve0(h, g_p, vps_p, cmd_p);
-		else
-			ret = set(h, veid, g_p, vps_p, cmd_p);
-		if (cmd_p->opt.save == YES) {
-			if (ret) {
-				logger(-1, 0, "Error: failed to apply "
-						"some parameters, not saving "
-						"configuration file!");
-				break;
-			}
-			get_vps_conf_path(veid, fname, sizeof(fname));
-			/* Warn if config does not exist */
-			if (!stat_file(fname))
-				logger(-1, errno, "WARNING: %s not found",
-					fname);
-			ret = vps_save_config(veid, fname,
-					cmd_p, vps_p, &g_action);
-		} else if (cmd_p->opt.save != NO) {
-			if (list_empty(&cmd_p->res.misc.userpw)) {
-				int is_run = h != NULL && vps_is_run(h, veid);
-				logger(0, 0, "WARNING: Settings were not saved"
-				" to config %s(use --save flag)",
-				is_run ? "and will be lost after CT restart "
-					: "");
-			}
-		}
+		ret = vps_set(h, veid, g_p, vps_p, cmd_p, argc-1, argv+1);
 		break;
 	case ACTION_STATUS:
 		ret = show_status(h, veid, g_p);
@@ -1512,6 +1620,14 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 		ret = vzctl_env_snapshot_list(argc, argv, veid,
 				g_p->res.fs.private);
 		break;
+	case ACTION_SNAPSHOT_MOUNT:
+		ret = vzctl_env_mount_snapshot(veid, g_p->res.fs.private,
+				cmd_p->snap.target, cmd_p->snap.guid);
+		break;
+	case ACTION_SNAPSHOT_UMOUNT:
+		ret = vzctl_env_umount_snapshot(veid, g_p->res.fs.private,
+				cmd_p->snap.guid);
+		break;
 #endif
 	case ACTION_CUSTOM:
 		ret = mod_setup(h, veid, 0, 0, &g_action, g_p);
