@@ -65,6 +65,7 @@ static void usage(int rc)
 "vzctl create <ctid> [--ostemplate <name>] [--config <name>]\n"
 "   [--layout ploop|simfs] [--hostname <name>] [--name <name>] [--ipadd <addr>]\n"
 "   [--diskspace <kbytes>] [--private <path>] [--root <path>]\n"
+"   [--local_uid <UID>] [--local_gid <GID>]\n"
 "vzctl start <ctid> [--force] [--wait]\n"
 "vzctl destroy | mount | umount | stop | restart | status <ctid>\n"
 #ifdef HAVE_PLOOP
@@ -73,6 +74,8 @@ static void usage(int rc)
 "vzctl snapshot <ctid> [--id <uuid>] [--name <name>] [--description <desc>]\n"
 "   [--skip-suspend]\n"
 "vzctl snapshot-switch | snapshot-delete <ctid> --id <uuid>\n"
+"vzctl snapshot-mount <ctid> --id <uuid> --target <dir>\n"
+"vzctl snapshot-umount <ctid> --id <uuid>\n"
 "vzctl snapshot-list <ctid> [-H] [-o field[,field...]] [--id <uuid>]\n"
 #endif
 "vzctl quotaon | quotaoff | quotainit <ctid>\n"
@@ -227,6 +230,10 @@ int main(int argc, char *argv[], char *envp[])
 		action = ACTION_SNAPSHOT_DELETE;
 	} else if (!strcmp(argv[1], "snapshot-list")) {
 		action = ACTION_SNAPSHOT_LIST;
+	} else if (!strcmp(argv[1], "snapshot-mount")) {
+		action = ACTION_SNAPSHOT_MOUNT;
+	} else if (!strcmp(argv[1], "snapshot-umount")) {
+		action = ACTION_SNAPSHOT_UMOUNT;
 #endif
 	} else if (!strcmp(argv[1], "--help")) {
 		usage(0);
