@@ -475,9 +475,8 @@ static void print_vswap(struct Cveinfo *p, int index)
 {
 	/* Same conditions as used in is_vswap_config() */
 	int vswap = (p->ubc != NULL) &&
-		(p->ubc->physpages != NULL) &&
-		(p->ubc->physpages[1] != LONG_MAX) &&
-		(p->ubc->physpages[1] != INT_MAX);
+		(p->ubc->physpages[3] != LONG_MAX) &&
+		(p->ubc->physpages[3] != INT_MAX);
 
 	print_bool("%5s", vswap);
 }
@@ -1571,7 +1570,7 @@ static int get_mounted_status()
 			veinfo[i].status = VE_SUSPENDED;
 		if (veinfo[i].root == NULL)
 			continue;
-		if (vps_is_mounted(veinfo[i].root, veinfo[i].private))
+		if (vps_is_mounted(veinfo[i].root, veinfo[i].private) == 1)
 			veinfo[i].status = VE_MOUNTED;
 	}
 	return 0;
